@@ -20,8 +20,19 @@
 __all__ = ["__author__", "__description__", "__license__", "__url__",
            "__version__"]
 
+
+def git_hash():
+    import subprocess
+    try:
+        out = subprocess.check_output(["git", "rev-parse", "--short", "HEAD"],
+                                      stderr=subprocess.STDOUT)
+        return out.decode("utf-8").rstrip("\n")
+    except Exception:
+        return "git"
+
+
 __author__ = "Rémi Duraffort"
 __description__ = 'LAVA XML-RPC command line interface'
 __license__ = 'AGPLv3+'
 __url__ = 'https://git.linaro.org/lava/lavacli.git'
-__version__ = "0.7"
+__version__ = "0.8"
