@@ -17,10 +17,11 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with lavacli.  If not, see <http://www.gnu.org/licenses/>
 
+import json
 import yaml
 
 
-def configure_parser(parser):
+def configure_parser(parser, _):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -34,16 +35,24 @@ def configure_parser(parser):
 
     # "list"
     aliases_list = sub.add_parser("list", help="list available aliases")
-    aliases_list.add_argument("--yaml", dest="output_format", default=None,
-                              action="store_const", const="yaml",
-                              help="print as yaml")
+    out_format = aliases_list.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format", default=None,
+                            action="store_const", const="yaml",
+                            help="print as yaml")
 
     # "show"
     aliases_show = sub.add_parser("show", help="show alias details")
     aliases_show.add_argument("alias", help="alias")
-    aliases_show.add_argument("--yaml", dest="output_format",
-                              action="store_const", const="yaml",
-                              default=None, help="print as yaml")
+    out_format = aliases_show.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            default=None, help="print as yaml")
 
 
 def help_string():
@@ -60,7 +69,9 @@ def handle_delete(proxy, options):
 
 def handle_list(proxy, options):
     aliases = proxy.scheduler.aliases.list()
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        print(json.dumps(aliases))
+    elif options.output_format == "yaml":
         print(yaml.dump(aliases).rstrip("\n"))
     else:
         print("Aliases:")
@@ -70,7 +81,9 @@ def handle_list(proxy, options):
 
 def handle_show(proxy, options):
     alias = proxy.scheduler.aliases.show(options.alias)
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        print(json.dumps(alias))
+    elif options.output_format == "yaml":
         print(yaml.dump(alias).rstrip("\n"))
     else:
         print("name        : %s" % alias["name"])
