@@ -18,11 +18,14 @@
 # along with lavacli.  If not, see <http://www.gnu.org/licenses/>
 
 import argparse
+import json
 import time
 import yaml
 
+from lavacli.utils import VERSION_2018_1
 
-def configure_parser(parser):
+
+def configure_parser(parser, version):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -38,12 +41,21 @@ def configure_parser(parser):
     owner.add_argument("--user", default=None, help="device owner")
     owner.add_argument("--group", default=None, help="device group owner")
 
-    devices_add.add_argument("--health", default=None,
-                             choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
-                             help="device health")
+    if version >= VERSION_2018_1:
+        devices_add.add_argument("--health", default=None,
+                                 choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
+                                 help="device health")
+    else:
+        devices_add.add_argument("--status", default=None,
+                                 choices=["OFFLINE", "IDLE", "RUNNING",
+                                          "OFFLINING", "RETIRED", "RESERVED"],
+                                 help="device status")
+        devices_add.add_argument("--health", default=None,
+                                 choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
+                                 help="device health status")
 
     devices_add.add_argument("--private", action="store_true", default=False,
-                             help="private device, public by default")
+                             help="private device [default=public]")
 
     # "dict"
     devices_dict = sub.add_parser("dict", help="device dictionary")
@@ -71,27 +83,36 @@ def configure_parser(parser):
     devices_list.add_argument("--all", "-a", action="store_true",
                               default=False,
                               help="list every devices, inluding retired")
-    devices_list.add_argument("--yaml", dest="output_format", default=None,
-                              action="store_const", const="yaml",
-                              help="print as yaml")
+    out_format = devices_list.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format", default=None,
+                            action="store_const", const="yaml",
+                            help="print as yaml")
 
     # "maintenance"
-    devices_maintenance = sub.add_parser("maintenance",
-                                         help="maintenance the device")
-    devices_maintenance.add_argument("hostname", help="device hostname")
-    devices_maintenance.add_argument("--force", default=False,
-                                     action="store_true",
-                                     help="force device maintenance by canceling running job")
-    devices_maintenance.add_argument("--no-wait", dest="wait",
-                                     default=True, action="store_false",
-                                     help="do not wait for the device to be idle")
+    if version >= VERSION_2018_1:
+        devices_maintenance = sub.add_parser("maintenance",
+                                             help="maintenance the device")
+        devices_maintenance.add_argument("hostname", help="device hostname")
+        devices_maintenance.add_argument("--force", default=False,
+                                         action="store_true",
+                                         help="force device maintenance by canceling running job")
+        devices_maintenance.add_argument("--no-wait", dest="wait",
+                                         default=True, action="store_false",
+                                         help="do not wait for the device to be idle")
 
     # "show"
     devices_show = sub.add_parser("show", help="show device details")
     devices_show.add_argument("hostname", help="device hostname")
-    devices_show.add_argument("--yaml", dest="output_format",
-                              action="store_const", const="yaml",
-                              default=None, help="print as yaml")
+    out_format = devices_show.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            default=None, help="print as yaml")
 
     # "tags"
     devices_tags = sub.add_parser("tags", help="manage tags for the given device")
@@ -102,16 +123,20 @@ def configure_parser(parser):
     tags_add.add_argument("hostname", help="hostname of the device")
     tags_add.add_argument("tag", help="name of the tag")
 
-    tags_list = tags_sub.add_parser("list", help="list tags for the device")
-    tags_list.add_argument("hostname", help="hostname of the device")
-    tags_list.add_argument("--yaml", dest="output_format",
-                           action="store_const", const="yaml",
-                           help="print as yaml")
-
     tags_del = tags_sub.add_parser("delete", help="remove a tag")
     tags_del.add_argument("hostname", help="hostname of the device")
     tags_del.add_argument("tag", help="name of the tag")
 
+    tags_list = tags_sub.add_parser("list", help="list tags for the device")
+    tags_list.add_argument("hostname", help="hostname of the device")
+    out_format = tags_list.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            help="print as yaml")
+
     # "update"
     devices_update = sub.add_parser("update", help="update device properties")
     devices_update.add_argument("hostname",
@@ -125,9 +150,18 @@ def configure_parser(parser):
     owner.add_argument("--user", default=None, help="device owner")
     owner.add_argument("--group", default=None, help="device group owner")
 
-    devices_update.add_argument("--health", default=None,
-                                choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
-                                help="device health")
+    if version >= VERSION_2018_1:
+        devices_update.add_argument("--health", default=None,
+                                    choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
+                                    help="device health")
+    else:
+        devices_update.add_argument("--status", default=None,
+                                    choices=["OFFLINE", "IDLE", "RUNNING",
+                                             "OFFLINING", "RETIRED", "RESERVED"],
+                                    help="device status")
+        devices_update.add_argument("--health", default=None,
+                                    choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
+                                    help="device health status")
 
     display = devices_update.add_mutually_exclusive_group()
     display.add_argument("--public", default=None, action="store_true",
@@ -140,11 +174,18 @@ def help_string():
     return "manage devices"
 
 
-def handle_add(proxy, options):
-    proxy.scheduler.devices.add(options.hostname, options.type,
-                                options.worker, options.user,
-                                options.group, not options.private,
-                                options.health, options.description)
+def handle_add(proxy, options, config):
+    if config["version"] >= VERSION_2018_1:
+        proxy.scheduler.devices.add(options.hostname, options.type,
+                                    options.worker, options.user,
+                                    options.group, not options.private,
+                                    options.health, options.description)
+    else:
+        proxy.scheduler.devices.add(options.hostname, options.type,
+                                    options.worker, options.user,
+                                    options.group, not options.private,
+                                    options.status, options.health,
+                                    options.description)
 
 
 def _lookups(value, fields):
@@ -157,14 +198,14 @@ def _lookups(value, fields):
     except IndexError:
         print("list index out of range (%d vs %d)" % (int(key), len(value)))
         return 1
-    except KeyError as exc:
+    except KeyError:
         print("Unknow key '%s' for '%s'" % (key, value))
         return 1
     print(value)
     return 0
 
 
-def handle_dict(proxy, options):
+def handle_dict(proxy, options, _):
     if options.sub_sub_sub_command == "get":
         config = proxy.scheduler.devices.get_dictionary(options.hostname,
                                                         options.render,
@@ -204,18 +245,23 @@ def handle_dict(proxy, options):
         return 0 if ret else 1
 
 
-def handle_list(proxy, options):
+def handle_list(proxy, options, config):
     devices = proxy.scheduler.devices.list(options.all)
 
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        print(json.dumps(devices))
+    elif options.output_format == "yaml":
         print(yaml.dump(devices).rstrip("\n"))
     else:
         print("Devices:")
         for device in devices:
-            print("* %s (%s): %s,%s" % (device["hostname"], device["type"], device["state"], device["health"]))
+            if config["version"] >= VERSION_2018_1:
+                print("* %s (%s): %s,%s" % (device["hostname"], device["type"], device["state"], device["health"]))
+            else:
+                print("* %s (%s): %s" % (device["hostname"], device["type"], device["status"]))
 
 
-def handle_maintenance(proxy, options):
+def handle_maintenance(proxy, options, _):
     proxy.scheduler.devices.update(options.hostname, None, None, None, None,
                                    "MAINTENANCE", None)
     device = proxy.scheduler.devices.show(options.hostname)
@@ -231,16 +277,21 @@ def handle_maintenance(proxy, options):
             time.sleep(5)
 
 
-def handle_show(proxy, options):
+def handle_show(proxy, options, config):
     device = proxy.scheduler.devices.show(options.hostname)
 
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        print(json.dumps(device))
+    elif options.output_format == "yaml":
         print(yaml.dump(device).rstrip("\n"))
     else:
         print("name        : %s" % device["hostname"])
         print("device-type : %s" % device["device_type"])
-        print("state       : %s" % device["state"])
-        print("health      : %s" % device["health"])
+        if config["version"] >= VERSION_2018_1:
+            print("state       : %s" % device["state"])
+            print("health      : %s" % device["health"])
+        else:
+            print("status      : %s" % device["status"])
         print("user        : %s" % device["user"])
         print("group       : %s" % device["group"])
         print("health      : %s" % device["health"])
@@ -254,14 +305,16 @@ def handle_show(proxy, options):
         print("tags        : %s" % device["tags"])
 
 
-def handle_tags(proxy, options):
+def handle_tags(proxy, options, _):
     if options.sub_sub_sub_command == "add":
         proxy.scheduler.devices.tags.add(options.hostname, options.tag)
     elif options.sub_sub_sub_command == "delete":
         proxy.scheduler.devices.tags.delete(options.hostname, options.tag)
     else:
         tags = proxy.scheduler.devices.tags.list(options.hostname)
-        if options.output_format == "yaml":
+        if options.output_format == "json":
+            print(json.dumps(tags))
+        elif options.output_format == "yaml":
             print(yaml.dump(tags).rstrip("\n"))
         else:
             print("Tags:")
@@ -269,14 +322,20 @@ def handle_tags(proxy, options):
                 print("* %s" % tag)
 
 
-def handle_update(proxy, options):
-    proxy.scheduler.devices.update(options.hostname, options.worker,
-                                   options.user, options.group,
-                                   options.public, options.health,
-                                   options.description)
+def handle_update(proxy, options, config):
+    if config["version"] >= VERSION_2018_1:
+        proxy.scheduler.devices.update(options.hostname, options.worker,
+                                       options.user, options.group,
+                                       options.public, options.health,
+                                       options.description)
+    else:
+        proxy.scheduler.devices.update(options.hostname, options.worker,
+                                       options.user, options.group,
+                                       options.public, options.status,
+                                       options.health, options.description)
 
 
-def handle(proxy, options, _):
+def handle(proxy, options, config):
     handlers = {
         "add": handle_add,
         "dict": handle_dict,
@@ -286,4 +345,4 @@ def handle(proxy, options, _):
         "tags": handle_tags,
         "update": handle_update
     }
-    return handlers[options.sub_sub_command](proxy, options)
+    return handlers[options.sub_sub_command](proxy, options, config)
