@@ -22,8 +22,10 @@ from urllib.parse import urlparse
 import zmq
 from zmq.utils.strtypes import b, u
 
+from lavacli.utils import VERSION_2018_1
 
-def configure_parser(parser):
+
+def configure_parser(parser, version):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -31,41 +33,42 @@ def configure_parser(parser):
     sub.add_parser("listen", help="listen to events")
 
     # "wait"
-    wait_parser = sub.add_parser("wait", help="wait for a specific event")
-    obj_parser = wait_parser.add_subparsers(dest="object", help="object to wait")
-    obj_parser.required = True
+    if version >= VERSION_2018_1:
+        wait_parser = sub.add_parser("wait", help="wait for a specific event")
+        obj_parser = wait_parser.add_subparsers(dest="object", help="object to wait")
+        obj_parser.required = True
 
-    # "wait device"
-    device_parser = obj_parser.add_parser("device")
-    device_parser.add_argument("hostname", type=str,
-                               help="hostname of the device")
-    device_parser.add_argument("--state", default=None,
-                               choices=["IDLE", "RESERVED", "RUNNING"],
-                               help="device state")
-    device_parser.add_argument("--health", default=None,
-                               choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
-                               help="device health")
+        # "wait device"
+        device_parser = obj_parser.add_parser("device")
+        device_parser.add_argument("hostname", type=str,
+                                   help="hostname of the device")
+        device_parser.add_argument("--state", default=None,
+                                   choices=["IDLE", "RESERVED", "RUNNING"],
+                                   help="device state")
+        device_parser.add_argument("--health", default=None,
+                                   choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
+                                   help="device health")
 
-    # "wait job"
-    testjob_parser = obj_parser.add_parser("job")
-    testjob_parser.add_argument("job_id", help="job id")
-    testjob_parser.add_argument("--state", default=None,
-                                choices=["SUBMITTED", "SCHEDULING", "SCHEDULED", "RUNNING", "CANCELING", "FINISHED"],
-                                help="job state")
-    testjob_parser.add_argument("--health", default=None,
-                                choices=["UNKNOWN", "COMPLETE", "INCOMPLETE", "CANCELED"],
-                                help="job health")
+        # "wait job"
+        testjob_parser = obj_parser.add_parser("job")
+        testjob_parser.add_argument("job_id", help="job id")
+        testjob_parser.add_argument("--state", default=None,
+                                    choices=["SUBMITTED", "SCHEDULING", "SCHEDULED", "RUNNING", "CANCELING", "FINISHED"],
+                                    help="job state")
+        testjob_parser.add_argument("--health", default=None,
+                                    choices=["UNKNOWN", "COMPLETE", "INCOMPLETE", "CANCELED"],
+                                    help="job health")
 
-    # "wait worker"
-    worker_parser = obj_parser.add_parser("worker")
-    worker_parser.add_argument("hostname", type=str,
-                               help="worker hostname")
-    worker_parser.add_argument("--state", default=None,
-                               choices=["ONLINE", "OFFLINE"],
-                               help="worker state")
-    worker_parser.add_argument("--health", default=None,
-                               choices=["ACTIVE", "MAINTENANCE", "RETIRED"],
-                               help="worker health")
+        # "wait worker"
+        worker_parser = obj_parser.add_parser("worker")
+        worker_parser.add_argument("hostname", type=str,
+                                   help="worker hostname")
+        worker_parser.add_argument("--state", default=None,
+                                   choices=["ONLINE", "OFFLINE"],
+                                   help="worker state")
+        worker_parser.add_argument("--health", default=None,
+                                   choices=["ACTIVE", "MAINTENANCE", "RETIRED"],
+                                   help="worker health")
 
 
 def help_string():
@@ -125,13 +128,20 @@ def handle_listen(proxy, options, config):
         # Print according to the topic
         topic_end = topic.split(".")[-1]
         if topic_end == "device":
-            msg = "[%s] <%s> state=%s health=%s" % (data["device"], data["device_type"], data["state"], data["health"])
+            if config["version"] >= VERSION_2018_1:
+                msg = "[%s] <%s> state=%s health=%s" % (data["device"], data["device_type"], data["state"], data["health"])
+            else:
+                msg = "[%s] <%s> %s" % (data["device"], data["device_type"], data["status"])
             if "job" in data:
                 msg += " for %s" % data["job"]
         elif topic_end == "testjob":
-            msg = "[%s] <%s> state=%s health=%s (%s)" % (data["job"], data.get("device", "??"),
-                                                         data["state"], data["health"],
-                                                         data["description"])
+            if config["version"] >= VERSION_2018_1:
+                msg = "[%s] <%s> state=%s health=%s (%s)" % (data["job"], data.get("device", "??"),
+                                                             data["state"], data["health"],
+                                                             data["description"])
+            else:
+                msg = "[%s] <%s> %s (%s)" % (data["job"], data.get("device", "??"),
+                                             data["status"], data["description"])
         elif topic_end == "worker":
             msg = "[%s] state=%s health=%s" % (data["hostname"], data["state"], data["health"])
         print("\033[1;30m%s\033[0m \033[1;37m%s\033[0m \033[32m%s\033[0m - %s" % (dt, topic, username, msg))
@@ -173,7 +183,7 @@ def handle_wait(proxy, options, config):
     while True:
         msg = sock.recv_multipart()
         try:
-            (topic, _, dt, username, data) = (u(m) for m in msg)
+            (topic, _uuid, _dt, _username, data) = (u(m) for m in msg)
         except ValueError:
             print("Invalid message: %s" % msg)
             continue
