@@ -21,7 +21,11 @@ import os
 import yaml
 
 
-def configure_parser(parser):
+class ConfigurationError(Exception):
+    pass
+
+
+def configure_parser(parser, version):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -30,7 +34,11 @@ def configure_parser(parser):
     config_add.add_argument("id", type=str, help="identity")
     config_add.add_argument("--uri", type=str, required=True,
                             help="URI of the lava-server RPC endpoint")
-    config_add.add_argument("--proxy", type=str, default="",
+    config_add.add_argument("--username", type=str, default=None,
+                            help="api username")
+    config_add.add_argument("--token", type=str, default=None,
+                            help="api token")
+    config_add.add_argument("--proxy", type=str, default=None,
                             help="http proxy")
 
     # "delete"
@@ -44,6 +52,19 @@ def configure_parser(parser):
     config_show = sub.add_parser("show", help="show identity details")
     config_show.add_argument("id", type=str, help="identity")
 
+    # "version"
+    config_version = sub.add_parser("version", help="expected version")
+    vers_sub = config_version.add_subparsers(dest="sub_sub_sub_command",
+                                             help="Sub commands")
+    vers_sub.required = True
+    vers_check = vers_sub.add_parser("check",
+                                     help="check version")
+    vers_check.add_argument("id", type=str, help="identity")
+
+    vers_upgrade = vers_sub.add_parser("upgrade",
+                                       help="upgrade to server version")
+    vers_upgrade.add_argument("id", type=str, help="identity")
+
 
 def help_string():
     return "manage lavacli configuration"
@@ -56,9 +77,12 @@ def _load_configuration():
 
     try:
         with open(config_filename, "r", encoding="utf-8") as f_conf:
-            return yaml.load(f_conf.read())
+            data = yaml.load(f_conf.read())
+            if not isinstance(data, dict):
+                raise ConfigurationError("Invalid configuration file")
+            return data
     except (FileNotFoundError, KeyError, TypeError):
-        return None
+        return {}
 
 
 def _save_configuration(config):
@@ -75,6 +99,10 @@ def handle_add(proxy, options):
     config[options.id] = {"uri": options.uri}
     if options.proxy:
         config[options.id]["proxy"] = options.proxy
+    if options.username:
+        config[options.id]["username"] = options.username
+    if options.token:
+        config[options.id]["token"] = options.token
 
     _save_configuration(config)
 
@@ -102,11 +130,36 @@ def handle_show(proxy, options):
         return 1
 
 
+def handle_version(proxy, options):
+    if options.sub_sub_sub_command == "check":
+        config = _load_configuration()
+        if config.get(options.id) is None:
+            print("Unknown identity '%s'" % options.id)
+            return 1
+
+        config_version = config[options.id].get("version")
+        server_version = proxy.system.version()
+
+        print("config: %s" % config_version)
+        print("server: %s" % server_version)
+        return config_version != server_version
+    else:
+        config = _load_configuration()
+        if config.get(options.id) is None:
+            print("Unknown identity '%s'" % options.id)
+            return 1
+        version = proxy.system.version()
+        config[options.id]["version"] = version
+        print("server: %s" % version)
+        _save_configuration(config)
+
+
 def handle(proxy, options, _):
     handlers = {
         "add": handle_add,
         "delete": handle_delete,
         "list": handle_list,
-        "show": handle_show
+        "show": handle_show,
+        "version": handle_version
     }
     return handlers[options.sub_sub_command](proxy, options)
