@@ -19,14 +19,20 @@
 
 import argparse
 import datetime
+import json
+import pathlib
 import sys
 import time
+from urllib.parse import urlparse
 import yaml
 
-from lavacli.utils import print_u
+from lavacli.utils import (
+    print_u,
+    VERSION_2018_1,
+    VERSION_LATEST)
 
 
-def configure_parser(parser):
+def configure_parser(parser, version):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -34,18 +40,37 @@ def configure_parser(parser):
     jobs_cancel = sub.add_parser("cancel", help="cancel a job")
     jobs_cancel.add_argument("job_id", help="job id")
 
+    if version >= VERSION_LATEST:
+        # "config"
+        jobs_config = sub.add_parser("config", help="job configuration")
+        jobs_config.add_argument("job_id", help="job id")
+        jobs_config.add_argument("--dest", default=".",
+                                 help="save files into this directory")
+
+    # "definition"
     jobs_definition = sub.add_parser("definition", help="job definition")
     jobs_definition.add_argument("job_id", help="job id")
 
     # "list"
     jobs_list = sub.add_parser("list", help="list jobs")
+    if version >= VERSION_LATEST:
+        jobs_list.add_argument("--state", type=str, default=None,
+                               choices=["SUBMITTED", "SCHEDULING", "SCHEDULED", "RUNNING", "CANCELING", "FINISHED"],
+                               help="filter jobs by state")
+        jobs_list.add_argument("--health", type=str, default=None,
+                               choices=["UNKNOWN", "COMPLETE", "INCOMPLETE", "CANCELED"],
+                               help="filter jobs by health")
     jobs_list.add_argument("--start", type=int, default=0,
-                           help="skip the N first jobs")
-    jobs_list.add_argument("--limit", type=int, default=10,
-                           help="limit to N jobs")
-    jobs_list.add_argument("--yaml", dest="output_format", default=None,
-                           action="store_const", const="yaml",
-                           help="print as yaml")
+                           help="skip the N first jobs [default=0]")
+    jobs_list.add_argument("--limit", type=int, default=25,
+                           help="limit to N jobs [default=25]")
+    out_format = jobs_list.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format", default=None,
+                            action="store_const", const="yaml",
+                            help="print as yaml")
 
     # "logs"
     jobs_logs = sub.add_parser("logs", help="get logs")
@@ -55,30 +80,10 @@ def configure_parser(parser):
     jobs_logs.add_argument("--filters", default=None, type=str,
                            help="comma seperated list of levels to show")
     jobs_logs.add_argument("--polling", default=5, type=int,
-                           help="polling interval in seconds, 5s by default")
+                           help="polling interval in seconds [default=5s]")
     jobs_logs.add_argument("--raw", default=False, action="store_true",
                            help="print raw logs")
 
-    # "run"
-    jobs_run = sub.add_parser("run", help="run the job")
-    jobs_run.add_argument("definition", type=argparse.FileType('r'),
-                          help="job definition")
-    jobs_run.add_argument("--filters", default=None, type=str,
-                          help="comma seperated list of levels to show")
-    jobs_run.add_argument("--no-follow", default=False, action="store_true",
-                          help="do not keep polling until the end of the job")
-    jobs_run.add_argument("--polling", default=5, type=int,
-                          help="polling interval in seconds, 5s by default")
-    jobs_run.add_argument("--raw", default=False, action="store_true",
-                          help="print raw logs")
-
-    # "show"
-    jobs_show = sub.add_parser("show", help="job details")
-    jobs_show.add_argument("job_id", help="job id")
-    jobs_show.add_argument("--yaml", dest="output_format",
-                           action="store_const", const="yaml",
-                           default=None, help="print as yaml")
-
     # "resubmit"
     jobs_resubmit = sub.add_parser("resubmit", help="resubmit a job")
     jobs_resubmit.add_argument("job_id", help="job id")
@@ -88,44 +93,108 @@ def configure_parser(parser):
                                action="store_false",
                                help="resubmit and poll for the logs")
     jobs_resubmit.add_argument("--polling", default=5, type=int,
-                               help="polling interval in seconds, 5s by default")
+                               help="polling interval in seconds [default=5s]")
     jobs_resubmit.add_argument("--raw", default=False, action="store_true",
                                help="print raw logs")
 
+    # "run"
+    jobs_run = sub.add_parser("run", help="run the job")
+    jobs_run.add_argument("definition", type=argparse.FileType('r'),
+                          help="job definition")
+    jobs_run.add_argument("--filters", default=None, type=str,
+                          help="comma seperated list of levels to show")
+    jobs_run.add_argument("--no-follow", default=False, action="store_true",
+                          help="do not keep polling until the end of the job")
+    jobs_run.add_argument("--polling", default=5, type=int,
+                          help="polling interval in seconds [default=5s]")
+    jobs_run.add_argument("--raw", default=False, action="store_true",
+                          help="print raw logs")
+
+    # "show"
+    jobs_show = sub.add_parser("show", help="job details")
+    jobs_show.add_argument("job_id", help="job id")
+    out_format = jobs_show.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            default=None, help="print as yaml")
+
     # "submit"
     jobs_submit = sub.add_parser("submit", help="submit a new job")
     jobs_submit.add_argument("definition", type=argparse.FileType('r'),
                              help="job definition")
+    jobs_submit.add_argument("--url", action="store_true",
+                             dest="print_url", default=False,
+                             help="Print the full url")
 
-    # "wait"
-    jobs_wait = sub.add_parser("wait", help="wait for the job to finish")
-    jobs_wait.add_argument("job_id", help="job id")
-    jobs_wait.add_argument("--polling", default=5, type=int,
-                           help="polling interval in seconds, 5s by default")
+    if version >= VERSION_2018_1:
+        # "wait"
+        jobs_wait = sub.add_parser("wait", help="wait for the job to finish")
+        jobs_wait.add_argument("job_id", help="job id")
+        jobs_wait.add_argument("--polling", default=5, type=int,
+                               help="polling interval in seconds [default=5s]")
 
 
 def help_string():
     return "manage jobs"
 
 
-def handle_cancel(proxy, options):
+def handle_cancel(proxy, options, _):
     proxy.scheduler.jobs.cancel(options.job_id)
 
 
-def handle_definition(proxy, options):
+def handle_config(proxy, options, _):
+    config = proxy.scheduler.jobs.configuration(options.job_id)
+    definition, device, dispatcher, env, env_dut = config
+
+    # Create the directory if needed
+    dest = pathlib.Path(options.dest)
+    dest.mkdir(mode=0o755, exist_ok=True)
+
+    # Save the files
+    with open(dest / "definition.yaml", "w") as f_out:
+        f_out.write(definition)
+    if device is not None:
+        with open(dest / "device.yaml", "w") as f_out:
+            f_out.write(device)
+    if dispatcher is not None:
+        with open(dest / "dispatcher.yaml", "w") as f_out:
+            f_out.write(dispatcher)
+    if env is not None:
+        with open(dest / "env.yaml", "w") as f_out:
+            f_out.write(env)
+    if env_dut is not None:
+        with open(dest / "env.dut.yaml", "w") as f_out:
+            f_out.write(env_dut)
+
+
+def handle_definition(proxy, options, _):
     print(proxy.scheduler.jobs.definition(options.job_id))
 
 
-def handle_list(proxy, options):
-    jobs = proxy.scheduler.jobs.list(options.start, options.limit)
-    if options.output_format == "yaml":
+def handle_list(proxy, options, config):
+    if config["version"] >= VERSION_LATEST:
+        jobs = proxy.scheduler.jobs.list(options.state, options.health,
+                                         options.start, options.limit)
+    else:
+        jobs = proxy.scheduler.jobs.list(options.start, options.limit)
+    if options.output_format == "json":
+        print(json.dumps(jobs))
+    elif options.output_format == "yaml":
         print(yaml.dump(jobs).rstrip("\n"))
     else:
-        print("Jobs:")
+        print("Jobs (from %s to %s):" % (1 + options.start, options.start + options.limit))
         for job in jobs:
-            print("* %s: %s,%s [%s] (%s)" % (job["id"], job["state"],
-                                             job["health"], job["submitter"],
-                                             job["description"]))
+            if config["version"] >= VERSION_2018_1:
+                print("* %s: %s,%s [%s] (%s) - %s" % (job["id"], job["state"],
+                                                      job["health"], job["submitter"],
+                                                      job["description"], job["device_type"]))
+            else:
+                print("* %s: %s [%s] (%s) - %s" % (job["id"], job["status"],
+                                                   job["submitter"], job["description"],
+                                                   job["device_type"]))
 
 
 if sys.stdout.isatty():
@@ -184,7 +253,7 @@ def print_logs(logs, raw, filters):
                     COLORS[level] + msg + COLORS["end"])
 
 
-def handle_logs(proxy, options):
+def handle_logs(proxy, options, _):
     # Loop
     lines = 0
     while True:
@@ -209,7 +278,7 @@ def handle_logs(proxy, options):
                        options.raw, options.filters)
 
 
-def handle_resubmit(proxy, options):
+def handle_resubmit(proxy, options, config):
     job_id = proxy.scheduler.jobs.resubmit(options.job_id)
 
     if options.no_follow:
@@ -234,13 +303,13 @@ def handle_resubmit(proxy, options):
                              "msg": "[lavacli] Seeing %s logs" % job}],
                            options.raw, options.filters)
                 options.job_id = job
-                handle_logs(proxy, options)
+                handle_logs(proxy, options, config)
         else:
             options.job_id = job_id
-            handle_logs(proxy, options)
+            handle_logs(proxy, options, config)
 
 
-def handle_run(proxy, options):
+def handle_run(proxy, options, config):
     job_id = proxy.scheduler.jobs.submit(options.definition.read())
     print_logs([{"dt": datetime.datetime.utcnow().isoformat(),
                  "lvl": "info",
@@ -256,16 +325,21 @@ def handle_run(proxy, options):
                          "msg": "[lavacli] Seeing %s logs" % job}],
                        options.raw, options.filters)
             options.job_id = job
-            handle_logs(proxy, options)
+            handle_logs(proxy, options, config)
     else:
         options.job_id = job_id
-        handle_logs(proxy, options)
+        handle_logs(proxy, options, config)
 
 
-def handle_show(proxy, options):
+def handle_show(proxy, options, config):
     job = proxy.scheduler.jobs.show(options.job_id)
 
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        job["submit_time"] = job["submit_time"].value
+        job["start_time"] = job["start_time"].value
+        job["end_time"] = job["end_time"].value
+        print(json.dumps(job))
+    elif options.output_format == "yaml":
         job["submit_time"] = job["submit_time"].value
         job["start_time"] = job["start_time"].value
         job["end_time"] = job["end_time"].value
@@ -277,8 +351,11 @@ def handle_show(proxy, options):
         print("device-type : %s" % job["device_type"])
         print("device      : %s" % job["device"])
         print("health-check: %s" % job["health_check"])
-        print("state       : %s" % job["state"])
-        print("Health      : %s" % job["health"])
+        if config["version"] >= VERSION_2018_1:
+            print("state       : %s" % job["state"])
+            print("Health      : %s" % job["health"])
+        else:
+            print("status      : %s" % job["status"])
         if job.get("failure_comment"):
             print("failure     : %s" % job["failure_comment"])
         print("pipeline    : %s" % job["pipeline"])
@@ -289,16 +366,23 @@ def handle_show(proxy, options):
         print("end time    : %s" % job["end_time"])
 
 
-def handle_submit(proxy, options):
+def handle_submit(proxy, options, _):
     job_id = proxy.scheduler.jobs.submit(options.definition.read())
+
+    prefix = ""
+    if options.print_url:
+        parsed = urlparse(options.uri)
+        host = parsed.netloc.split("@")[-1]
+        prefix = "%s://%s/scheduler/job/" % (parsed.scheme, host)
+
     if isinstance(job_id, list):
         for job in job_id:
-            print(job)
+            print(prefix + str(job))
     else:
-        print(job_id)
+        print(prefix + str(job_id))
 
 
-def handle_wait(proxy, options):
+def handle_wait(proxy, options, _):
     job = proxy.scheduler.jobs.show(options.job_id)
     old_state = ""
     while job["state"] != "Finished":
@@ -314,9 +398,10 @@ def handle_wait(proxy, options):
         job = proxy.scheduler.jobs.show(options.job_id)
 
 
-def handle(proxy, options, _):
+def handle(proxy, options, config):
     handlers = {
         "cancel": handle_cancel,
+        "config": handle_config,
         "definition": handle_definition,
         "list": handle_list,
         "logs": handle_logs,
@@ -326,4 +411,4 @@ def handle(proxy, options, _):
         "submit": handle_submit,
         "wait": handle_wait,
     }
-    return handlers[options.sub_sub_command](proxy, options)
+    return handlers[options.sub_sub_command](proxy, options, config)
