@@ -23,7 +23,7 @@ import yaml
 from . import jobs
 
 
-def configure_parser(parser):
+def configure_parser(parser, _):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -48,7 +48,7 @@ def help_string():
     return "utility functions"
 
 
-def handle_logs(proxy, options):
+def handle_logs(_, options):
     try:
         logs = yaml.load(options.filename.read(), Loader=yaml.CLoader)
     except yaml.YAMLError:
