@@ -18,11 +18,14 @@
 # along with lavacli.  If not, see <http://www.gnu.org/licenses/>
 
 import argparse
+import json
 import time
 import yaml
 
+from lavacli.utils import VERSION_2017_12, VERSION_2018_1
 
-def configure_parser(parser):
+
+def configure_parser(parser, version):
     sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
     sub.required = True
 
@@ -54,9 +57,13 @@ def configure_parser(parser):
 
     # "list"
     workers_list = sub.add_parser("list", help="list workers")
-    workers_list.add_argument("--yaml", dest="output_format",
-                              action="store_const", const="yaml",
-                              default=None, help="print as yaml")
+    out_format = workers_list.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            default=None, help="print as yaml")
 
     # "maintenance"
     workers_maintenance = sub.add_parser("maintenance",
@@ -76,29 +83,42 @@ def configure_parser(parser):
                                help="worker hostname")
     update_parser.add_argument("--description", type=str, default=None,
                                help="worker description")
-    update_parser.add_argument("--health", type=str, default=None,
-                               choices=["ACTIVE", "MAINTENANCE", "RETIRED"],
-                               help="worker health")
+    if version >= VERSION_2017_12:
+        update_parser.add_argument("--health", type=str, default=None,
+                                   choices=["ACTIVE", "MAINTENANCE", "RETIRED"],
+                                   help="worker health")
+    else:
+        display = update_parser.add_mutually_exclusive_group()
+        display.add_argument("--disable", action="store_false",
+                             default=None, dest="display",
+                             help="disable the worker")
+        display.add_argument("--enable", action="store_true",
+                             default=None, dest="display",
+                             help="enable the worker")
 
     # "show"
     workers_show = sub.add_parser("show", help="show worker details")
     workers_show.add_argument("hostname", help="worker hostname")
-    workers_show.add_argument("--yaml", dest="output_format",
-                              action="store_const", const="yaml",
-                              default=None, help="print as yaml")
+    out_format = workers_show.add_mutually_exclusive_group()
+    out_format.add_argument("--json", dest="output_format", default=None,
+                            action="store_const", const="json",
+                            help="print as json")
+    out_format.add_argument("--yaml", dest="output_format",
+                            action="store_const", const="yaml",
+                            default=None, help="print as yaml")
 
 
 def help_string():
     return "manage workers"
 
 
-def handle_add(proxy, options):
+def handle_add(proxy, options, _):
     proxy.scheduler.workers.add(options.hostname,
                                 options.description,
                                 options.disabled)
 
 
-def handle_config(proxy, options):
+def handle_config(proxy, options, _):
     if options.sub_sub_sub_command == "get":
         config = proxy.scheduler.workers.get_config(options.hostname)
         print(str(config).rstrip("\n"))
@@ -111,9 +131,11 @@ def handle_config(proxy, options):
             return 1
 
 
-def handle_list(proxy, options):
+def handle_list(proxy, options, _):
     workers = proxy.scheduler.workers.list()
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        print(json.dumps(workers))
+    elif options.output_format == "yaml":
         print(yaml.dump(workers).rstrip("\n"))
     else:
         print("Workers:")
@@ -121,7 +143,7 @@ def handle_list(proxy, options):
             print("* %s" % worker)
 
 
-def handle_maintenance(proxy, options):
+def handle_maintenance(proxy, options, _):
     proxy.scheduler.workers.update(options.hostname, None, "MAINTENANCE")
 
     if options.force or options.wait:
@@ -141,26 +163,46 @@ def handle_maintenance(proxy, options):
                     time.sleep(5)
 
 
-def handle_show(proxy, options):
+def handle_show(proxy, options, config):
     worker = proxy.scheduler.workers.show(options.hostname)
-    if options.output_format == "yaml":
+    if options.output_format == "json":
+        if "last_ping" in worker:
+            worker["last_ping"] = worker["last_ping"].value
+        print(json.dumps(worker))
+    elif options.output_format == "yaml":
+        if "last_ping" in worker:
+            worker["last_ping"] = worker["last_ping"].value
         print(yaml.dump(worker).rstrip("\n"))
     else:
         print("hostname    : %s" % worker["hostname"])
         print("description : %s" % worker["description"])
-        print("state       : %s" % worker["state"])
-        print("health      : %s" % worker["health"])
-        print("devices     : %s" % ", ".join(worker["devices"]))
-        print("last ping   : %s" % worker["last_ping"])
+
+        if config["version"] >= VERSION_2017_12:
+            print("state       : %s" % worker["state"])
+            print("health      : %s" % worker["health"])
+        else:
+            print("master      : %s" % worker["master"])
+            print("hidden      : %s" % worker["hidden"])
+
+        if config["version"] >= VERSION_2018_1:
+            print("devices     : %s" % ", ".join(worker["devices"]))
+            print("last ping   : %s" % worker["last_ping"])
+        else:
+            print("devices     : %d" % worker["devices"])
 
 
-def handle_update(proxy, options):
-    proxy.scheduler.workers.update(options.hostname,
-                                   options.description,
-                                   options.health)
+def handle_update(proxy, options, config):
+    if config["version"] >= VERSION_2017_12:
+        proxy.scheduler.workers.update(options.hostname,
+                                       options.description,
+                                       options.health)
+    else:
+        proxy.scheduler.workers.update(options.hostname,
+                                       options.description,
+                                       not options.display)
 
 
-def handle(proxy, options, _):
+def handle(proxy, options, config):
     handlers = {
         "add": handle_add,
         "config": handle_config,
@@ -169,4 +211,4 @@ def handle(proxy, options, _):
         "show": handle_show,
         "update": handle_update
     }
-    return handlers[options.sub_sub_command](proxy, options)
+    return handlers[options.sub_sub_command](proxy, options, config)
